{ Testing  xMinSearchR1.pp
  See comments in this unit.
  --------------------------------------------}
program tEx066;

// To tell FreePascal that the application is a Graphical User Interface
// i.e. is running in windows, not in text mode.
{$APPTYPE GUI}

Uses

  wApplication0
// ,wWindowText0
 ,wWindow0
 ,xMinR1SearchR1
 ,wDialogCreatorLow2
 ,wVariableNumLink2
 ,wMessages2
 ,wGlobals2
 ,wDebug0
 ,Windows;

type
  PMyWindow = ^TMyWindow;
  TMyWindow = object(TWindow)
  //"""""""""""""""""""""""""
  oMinS : TMinR1SearchR1;
  // to minimise a function : R --> R

  nChoiceFunc : int32;
  // Choice of a function to minimise

  vXLeft,vXMiddle,vXRight : Float;
  // Starting values to minimise the function

  vTolErrAbs : Float;
  // Error tolerance on the minimum found

  nMaxIter : int32;
  // Maximum number of function evaluation

  strDlgTitle : string[80];
  // Title of the dialog box

  constructor Init(strWinTitlePrm:ShortString);
  destructor Done; virtual;

  procedure Test1;
  procedure Test2;
  procedure Test3;

  function WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  // Handles the menu messages

  function WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  end; // TMyWindow = object(TWindow)

var
  oApp        : TApplication;
  poMyMainWin : PMyWindow;

function func1(pData:pointer; vX:Float):Float;
//============================================
// One function to be minimised
begin
case int32(pData) of
   1 : func1:=1+sqr(vX);
   2 : func1:=2.0 - cos(vX);
   3 : func1:=1+sqr(vX)*(vX + 1.0);
   4 : func1:=exp(vX) - vX;
   5 : func1:=1+sqr(sqr(vX));
   6 : func1:=1+abs(vX);
   7 : func1:=1+sqrt(abs(vX));
   8 : func1:=1+sqr(sqr(vX)) + sqr(vX)*(vX+1);
   9 : func1:=1.0;
  10 : func1:=1+sqr(sin(vX) - vX);
  11 : func1:=1+sqr(sin(vX) - 1.1*vX);
   end; // Case

//WriteWin(0, 'eval at:' + FTS(vX));
end;  // func1

constructor TMyWindow.Init(strWinTitlePrm:ShortString);
//=====================================================
begin
inherited Init(strWinTitlePrm);

StrField(  1000, strDlgTitle, sizeOf(strDlgTitle));
int32Field(1001, nChoiceFunc);
FloatField(1002, vXLeft);
FloatField(1003, vXMiddle);
FloatField(1004, vXRight);
FloatField(1005, vTolErrAbs);
int32Field(1006, nMaxIter);

nChoiceFunc :=1;
vXLeft  :=-0.2;
vXMiddle:=0.15;
vXRight :=0.4;
vTolErrAbs :=1e-3;
nMaxIter   :=50;

// This line is all what is needed to have a status line.
new(poStatusLine, Init(@Self));
end;  // TMyWindow.Init

destructor TMyWindow.Done;
//========================
begin
inherited Done;
end;  // TMyWindow.Done

procedure TMyWindow.Test1;
//========================
// Test the SearchMinGoldenSection.
var
  nRep : integer;
begin
strDlgTitle:=GetMsg(20101);
nRep:=DialogExecMSG(20199, @Self);

if (nRep <> IDOK) then exit;

oMinS.Init;

// Data which must be given, before minimisation can start :
oMinS.func:=@func1;
oMinS.vXL :=vXLeft;
oMinS.vXM :=vXMiddle;
oMinS.vXR :=vXRight;

// This can be changed. Here default values are shown.
oMinS.vTolAbs   :=vTolErrAbs;
oMinS.nNbFuncEvalMax:=nMaxIter;
oMinS.pDataFunc :=pointer(nChoiceFunc);

// Do the minimisation
oMinS.SearchMinGoldenSection;

// Result of the minimisation
DBW(1, 'Number of function evaluation : ' + ITS5(oMinS.nNbFuncEval));
DBW(1, 'Minimum value :' + FTS(oMinS.vFSol));
DBW(1, 'Min found at : ' + FTS(oMinS.vXSol));
DBW(1, 'Better min at : ' + FloatToStr(oMinS.vXFMin,12,9));

oMinS.Done;
end;  // TMyWindow.Test1

procedure TMyWindow.Test2;
//========================
// Test the SearchMinBrent.
var
  nRep : integer;
begin
strDlgTitle:=GetMsg(20102);
nRep:=DialogExecMSG(20199, @Self);

if (nRep <> IDOK) then exit;

oMinS.Init;

// Data which must be given, before minimisation can start :
oMinS.func:=@func1;
oMinS.vXL :=vXLeft;
oMinS.vXM :=vXMiddle;
oMinS.vXR :=vXRight;

// This can be changed. Here default values are shown.
oMinS.vTolAbs   :=vTolErrAbs;
oMinS.nNbFuncEvalMax:=nMaxIter;
oMinS.pDataFunc :=pointer(nChoiceFunc);

// Do the minimisation
oMinS.SearchMinBrent;

// Result of the minimisation
DBW(1, 'Number of function evaluation : ' + ITS5(oMinS.nNbFuncEval));
DBW(1, 'Minimum value :' + FTS(oMinS.vFSol));
DBW(1, 'Min found at :      ' + FloatToStr(oMinS.vXSol,12,9));
DBW(1, 'Better min at :     ' + FloatToStr(oMinS.vXFMin,12,9));

oMinS.Done;
end;  // TMyWindow.Test2

procedure TMyWindow.Test3;
//========================
// To be used for another Test.
var
  nRep : integer;
begin
strDlgTitle:=GetMsg(20103);
nRep:=DialogExecMSG(20199, @Self);

if (nRep <> IDOK) then exit;

oMinS.Init;

// Data which must be given, before minimisation can start :
oMinS.func:=@func1;
oMinS.vXL :=vXLeft;
oMinS.vXM :=vXMiddle;
oMinS.vXR :=vXRight;

// This can be changed. Here default values are shown.
oMinS.vTolAbs   :=vTolErrAbs;
oMinS.nNbFuncEvalMax:=nMaxIter;
oMinS.pDataFunc :=pointer(nChoiceFunc);

// Bracket a minimum
oMinS.SearchMinBracket(0.1*(oMinS.vXR - oMinS.vXL));

// Result of the minimisation
DBW(1, 'ExitCode : ' + FTS(oMinS.nExitCode));
if (oMinS.nExitCode <> 0) then DBW(1, 'NO MINIMUM BRACKETED');
DBW(1, 'Number of function evaluation : ' + ITS5(oMinS.nNbFuncEval));
DBW(1, 'Bracket :' + FTS(oMinS.vXL) + FTS(oMinS.vXM) + FTS(oMinS.vXR));
DBW(1, 'Func=   :' + FTS(oMinS.vFL) + FTS(oMinS.vFM) + FTS(oMinS.vFR));
DBW(1, '"""""""""""""""""""""""""""""""""""""""""""""""""');
DBW(1, '');

if (oMinS.nExitCode = 0) then begin
   // Do the minimisation
   oMinS.wOptions:=7; // Function has been evaluated at vXL, vXM, vXR
   oMinS.SearchMinBrent;
   DBW(1, 'Number of function evaluation : ' + ITS5(oMinS.nNbFuncEval));
   DBW(1, 'Minimum value :' + FTS(oMinS.vFSol));
   DBW(1, 'Min found at :      ' + FloatToStr(oMinS.vXSol,12,9));
   DBW(1, 'Better min at :     ' + FloatToStr(oMinS.vXFMin,12,9));
   end;

oMinS.Done;
end;  // TMyWindow.Test3

function TMyWindow.WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT;
//=============================================================
// Handles the menu messages
Var wNrMenu     : word;
begin
WMCommand := 0; // Default return value when processing a message

if (lPar <> 0) then begin
   // call the parent WMCommand method.
   WMCommand:=inherited WMCommand(wPar, lPar);
   exit;
   end;

// for the menus,  lPar = 0
//=========================
wNrMenu   := LoWord(wPar);
Case wNrMenu Of
   101 : Test1;
   102 : Test2;
   103 : Test3;

   else WMCommand:=inherited WMCommand(wPar, lPar);
   end;  // Case NrMenu Of
end;  // TMyWindow.WMCommand

function TMyWindow.WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT;
//======================================================================================
begin
// Default treatment of a message. Does nearly nothing
WindowProc := inherited WindowProc(hWindow, wMsg, wPar, lPar);
end;  // TMyWindow.WindowProc

Begin // MAIN ******
// Name under which the main window is registered. Spaces are not allowed
oApp.Init('Example066');

// Creation of the main window
new(poMyMainWin, Init('tEx066. Testing xMinSearchR1.pp.'));

oApp.Run;
oApp.Done;  // The main window object is disposed here.
End.

