{ Testing  xMinSimplex.pp
  See comments in this unit.
  --------------------------------------------}
program tEx067;

// To tell FreePascal that the application is a Graphical User Interface
// i.e. is running in windows, not in text mode.
{$APPTYPE GUI}

Uses

  wApplication0
 ,wWindowText0
 ,wWindow0
 ,xMinRnSearchRN
 ,wDialogCreatorLow2
 ,wVariableNumLink2
 ,wMessages2
 ,wGlobals2
 ,wDebug0
 ,Windows;

type
  PMyWindow = ^TMyWindow;
  TMyWindow = object(TWindow)
  //"""""""""""""""""""""""""
  oMinS : TMinRNSearchRN;
  // to minimise a function : Rn --> R

  nChoiceFunc : int32;
  // Choice of a function to minimise

  vX1,vX2,vX3,vX4,vX5 : Float;
  // Starting guess minimum

  vXStartLen : Float;
  // Starting search length

  vTolErrAbs : Float;
  // Error tolerance on the minimum found

  nMaxFuncEval : int32;
  // Maximum number of function evaluation

  strDlgTitle : string[80];
  // Title of the dialog box

  constructor Init(strWinTitlePrm:ShortString);
  destructor Done; virtual;

  procedure Test1;
  procedure Test2;
  procedure Test3;

  function WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  // Handles the menu messages

  function WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  end; // TMyWindow = object(TWindow)

var
  oApp        : TApplication;
  poMyMainWin : PMyWindow;

function func1(pData:pointer; var avX:array of Float):Float;
//==========================================================
// One function to be minimised
var
  vMax        : Float;
  ii          : integer;
begin
case int32(pData) of
   1 : func1:=sqr(avX[1]-1) + sqr(avX[2]-2);
   2 : func1:=sqrt(1+sqr(avX[1]-1) + sqr(avX[2]-2));
   3 : begin
       vMax:=abs(avX[1]);
       for ii:=2 to 5 do if (vMax < abs(ii*avX[ii])) then vMax:=abs(ii*avX[ii]);

       // To have a minimum to find
       for ii:=1 to 5 do vMax:=vMax + 1e-3*sqr(avX[ii]-ii);
       func1:=vMax;
       end;
   4 : begin
       vMax:=0;
       for ii:=1 to 5 do vMax:=vMax+exp(sqr(avX[ii]));
       func1:=vMax;
       end;
   end; // case

WriteWin(0, 'eval at:' + FTS(avX[1]) + FTS(avX[2]) + FTS(avX[3]));
WriteWin(0, '        ' + FTS(avX[4]) + FTS(avX[5]) + FTS(func1));
end;  // func1

constructor TMyWindow.Init(strWinTitlePrm:ShortString);
//=====================================================
begin
inherited Init(strWinTitlePrm);

StrField(  1000, strDlgTitle, sizeOf(strDlgTitle));
int32Field(1001, nChoiceFunc);
int32Field(1002, nMaxFuncEval);

FloatField(1011, vTolErrAbs);
FloatField(1012, vXStartLen);

FloatField(1101, vX1); // Starting point
FloatField(1102, vX2);
FloatField(1103, vX3);
FloatField(1104, vX4);
FloatField(1105, vX5);

vX1:=3;  vX2:=3;  vX3:=3;  vX4:=3;  vX5:=3;
nChoiceFunc  :=1;
vTolErrAbs   :=1e-3;
nMaxFuncEval :=500;
vXStartLen   :=0.5;

// This line is all what is needed to have a status line.
new(poStatusLine, Init(@Self));
end;  // TMyWindow.Init

destructor TMyWindow.Done;
//========================
begin
inherited Done;
end;  // TMyWindow.Done

procedure TMyWindow.Test1;
//========================
// Minimisation using the simplex method
var
  nRep : integer;
begin
strDlgTitle:=GetMsg(20101);
nRep:=DialogExecMSG(20199, @Self);

if (nRep <> IDOK) then exit;

oMinS.Init(5);   // 5 = maximal dimension that can be given

case nChoiceFunc of
  1 : oMinS.nDim:=2;
  2 : oMinS.nDim:=2;
  3 : oMinS.nDim:=5;
  4 : oMinS.nDim:=5;
  end;

// Data which must be given, before minimisation can start :
// oMinS.nDim:= set in the init at the correct value
oMinS.func:=@func1;
oMinS.papvecV^[1]^[1]:=vX1; // Guess of initial values
oMinS.papvecV^[1]^[2]:=vX2;
oMinS.papvecV^[1]^[3]:=vX3;
oMinS.papvecV^[1]^[4]:=vX4;
oMinS.papvecV^[1]^[5]:=vX5;
oMinS.vSizeEdge:=vXStartLen;  // Starting size of the Simplex edges.

// This can be changed.
oMinS.vTolAbs   :=vTolErrAbs;
oMinS.vTolRel   :=0;
oMinS.nNbFuncEvalMax:=nMaxFuncEval;
oMinS.pDataFunc :=pointer(nChoiceFunc);

// change  oMinS.wOptions:=1; if you want to give initial values.
{oMinS.papvecV^[2]^[1]:=3;
oMinS.papvecV^[2]^[2]:=5;
oMinS.papvecV^[3]^[1]:=4;
oMinS.papvecV^[3]^[2]:=5; }

// Do the minimisation
oMinS.MinimiseSimplex;

// Result of the minimisation
WriteWin(0, 'Number of function evaluation : ' + ITS5(oMinS.nNbFuncEval));
WriteWin(0, 'Minimum value :' + FTS(oMinS.pavY^[oMinS.nLo]));
WriteWin(0, 'Min found at : ' + FTS(oMinS.papvecV^[oMinS.nLo]^[1])
                              + FTS(oMinS.papvecV^[oMinS.nLo]^[2]));

oMinS.Done;
end;  // TMyWindow.Test1

procedure TMyWindow.Test2;
//========================
// Minimisation using the Powell method
var
  nRep : integer;
begin
strDlgTitle:=GetMsg(20102);
nRep:=DialogExecMSG(20199, @Self);

if (nRep <> IDOK) then exit;

oMinS.Init(5);   // 5 = maximal dimension that can be given

case nChoiceFunc of
  1 : oMinS.nDim:=2;
  2 : oMinS.nDim:=2;
  3 : oMinS.nDim:=5;
  4 : oMinS.nDim:=5;
  end;

// Data which must be given, before minimisation can start :
// oMinS.nDim:= set in the init at the correct value
oMinS.func:=@func1;
oMinS.pvecXMin^[1]:=vX1; // Guess of initial values
oMinS.pvecXMin^[2]:=vX2;
oMinS.pvecXMin^[3]:=vX3;
oMinS.pvecXMin^[4]:=vX4;
oMinS.pvecXMin^[5]:=vX5;
oMinS.vSizeEdge:=vXStartLen;  // Starting size of the Simplex edges.

// This can be changed.
//oMinS.wOptions  :=0;
oMinS.vTolAbs   :=vTolErrAbs;
oMinS.vTolRel   :=0;
oMinS.nNbFuncEvalMax:=nMaxFuncEval;
oMinS.pDataFunc :=pointer(nChoiceFunc);

// change  oMinS.wOptions:=1; if you want to give initial values.
{oMinS.papvecV^[2]^[1]:=0.5;
oMinS.papvecV^[2]^[2]:= 0.5;
oMinS.papvecV^[3]^[1]:= 0.5;
oMinS.papvecV^[3]^[2]:=-0.5; }

// Do the minimisation
oMinS.MinimisePowell;

// Result of the minimisation
WriteWin(0, 'Number of function evaluation : ' + ITS5(oMinS.nNbFuncEval));
WriteWin(0, 'Minimum value :' + FTS(oMinS.vFXMin));
WriteWin(0, 'Min found at : ' + FTS(oMinS.pvecXMin^[1])
                              + FTS(oMinS.pvecXMin^[2]));

oMinS.Done;
end;  // TMyWindow.Test2

procedure TMyWindow.Test3;
//========================
var
  vOne,
  gglvFloatEpsilon : Float;
begin
gglvFloatEpsilon:=1;
repeat
   gglvFloatEpsilon:=0.5*gglvFloatEpsilon;
   vOne:=1.0 + gglvFloatEpsilon;
   until (vOne = 1);

DBW(1, FloatToStr(gglvFloatEpsilon*1e18,18,15));
end;  // TMyWindow.Test3

function TMyWindow.WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT;
//=============================================================
// Handles the menu messages
Var wNrMenu     : word;
begin
WMCommand := 0; // Default return value when processing a message

if (lPar <> 0) then begin
   // call the parent WMCommand method.
   WMCommand:=inherited WMCommand(wPar, lPar);
   exit;
   end;

// for the menus,  lPar = 0
//=========================
wNrMenu   := LoWord(wPar);
Case wNrMenu Of
   101 : Test1;
   102 : Test2;
   103 : Test3;

   else WMCommand:=inherited WMCommand(wPar, lPar);
   end;  // Case NrMenu Of
end;  // TMyWindow.WMCommand

function TMyWindow.WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT;
//======================================================================================
begin
// Default treatment of a message. Does nearly nothing
WindowProc := inherited WindowProc(hWindow, wMsg, wPar, lPar);
end;  // TMyWindow.WindowProc

Begin // MAIN ******
// Name under which the main window is registered. Spaces are not allowed
oApp.Init('Example067');

// Creation of the main window
new(poMyMainWin, Init('tEx067. Testing xMinSearchRN.pp.'));

oApp.Run;
oApp.Done;  // The main window object is disposed here.
End.

