{ Testing ideas for characterising non locality.

  --------------------------------------------}
program tzNonLocal01;

// To tell FreePascal that the application is a Graphical User Interface
// i.e. is running in windows, not in text mode.
{$APPTYPE GUI}

Uses

  wApplication0
 ,wWindow0
 ,xMinRNSearchRN
 ,xVectR3
 ,wGlobals2
 ,wDebug0
 ,Windows;

type
  TArray4 = array[1..4,1..4,1..4,1..4] of Float;
  TArray2 = array[1..4,1..4] of Float;
  PMyWindow = ^TMyWindow;
  TMyWindow = object(TWindow)
  //"""""""""""""""""""""""""
  avPr : TArray4; // Probabilities from the experimental results
  avP1, avP2 : TArray2; // Marginal probabilities
  avFF : TArray4; // Functions of the random variables of Alice and Bob
  iatt,ibtt,jatt,jbtt : integer;

  avecV : array[1..730] of array[1..100] of Float;
  // Array of vectors.
  // We are looking for a vector  X  such that  avecV * X  has all its
  // component below zero

  nNbLine,  // Number of vectors of avecV
  nNbCol    // dimension of the vectors of  avecV[ii]
            : int32;

  vecX : array[1..100] of Float;
  // searched vector.

  vMinFunc : Float;
  // Minimum of all scalar product to compute the function.
  // If this minimum is > 0, then the problem is non local

  constructor Init(strWinTitlePrm:ShortString);
  destructor Done; virtual;

  procedure Test1;
  procedure Test2;
  procedure Test3;
  procedure Test4;

  procedure SearchX;
  // Search a vector  X  such that  avecV[ii]*X < 0 for all X

  function WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  // Handles the menu messages

  function WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  end; // TMyWindow = object(TWindow)

var
  oApp        : TApplication;
  poMyMainWin : PMyWindow;
  glf         : boolean; // necessary to use a special assignment operator

constructor TMyWindow.Init(strWinTitlePrm:ShortString);
//=====================================================
begin
inherited Init(strWinTitlePrm);

// This line is all what is needed to have a status line.
new(poStatusLine, Init(@Self));
end;  // TMyWindow.Init

destructor TMyWindow.Done;
//=========================
begin
inherited Done;
end;  // TMyWindow.Done

procedure TMyWindow.Test1;
//========================
var
  iat,ibt,jat,jbt : integer;
  ia,ib,ja,jb     : integer;
  ja1,ja2,jb1,jb2 : integer;
  avThetaA,
  avThetaB    : array[1..5] of Float;
  avecSetA,
  avecSetB    : array[1..5] of RVectR3Float;
  vSign,
  vNoise,
  vLambda,
  vMean,
  vRealis     // One realisation
              : Float;
begin
iat:=2; // Number of settings
ibt:=2;
jat:=2;
jbt:=2;

// Defining the settings
avThetaA[1] := 0.0;
aVThetaA[2] := pi/2;
avThetaB[1] := pi/4;
aVThetaB[2] :=-pi/4;

for ia:=1 to iat do
   avecSetA[ia]:=V3Assign(0.0, cos(avThetaA[ia]), sin(avThetaA[ia]));

for ib:=1 to ibt do
   avecSetB[ib]:=V3Assign(0.0, cos(avThetaB[ib]), sin(avThetaB[ib]));

// Noise in the entangled state
vNoise:=1-1/sqrt(2); // = Limit = 0.292893219
vNoise:=0.292;
vLambda:=1.0 - vNoise;

// Defining the Probabilities
DBW(1, '--- Pr --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         if odd(ja+jb) then vSign:= 1
         else               vSign:=-1;
         avPr[ia,ib,ja,jb]:=0.25*(vLambda*(1.0 - vSign * avecSetA[ia]*avecSetB[ib]) + vNoise);

         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avPr[ia,ib,ja,jb]));
         end;

// Marginal probabilities on Alice side
DBW(1, '--- P1 --------');
for ia:=1 to iat do
    for ja:=1 to jat do begin
      avP1[ia,ja]:=0;
      for jb:=1 to jbt do avP1[ia,ja]:=avP1[ia,ja] + avPr[ia,1,ja,jb];

//      DBW(1, ITS5(ia) + ITS5(ja) + FTS(avP1[ia,ja]));
      end;

// Marginal probabilities on Bob side
DBW(1, '--- P2 --------');
for ib:=1 to ibt do
    for jb:=1 to jbt do begin
      avP2[ib,jb]:=0;
      for ja:=1 to jat do avP2[ib,jb]:=avP2[ib,jb] + avPr[1,ib,ja,jb];

//      DBW(1, ITS5(ib) + ITS5(jb) + FTS(avP2[ib,jb]));
      end;

// Try to define the function of the random variables
DBW(1, '--- FF --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
//         avFF[ia,ib,ja,jb]:=(avPr[ia,ib,ja,jb] - avP1[ia,ja] * avP2[ib,jb]);
         avFF[ia,ib,ja,jb]:=avPr[ia,ib,ja,jb];

         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avFF[ia,ib,ja,jb]));
         end;

// Esperance of the combination of the random variables
vMean:=0;
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         vMean:=vMean + avFF[ia,ib,ja,jb] * avPr[ia,ib,ja,jb];
         end;

// Comparaison between the Esperance and the realisations.
DBW(1, '-- Mean  v.s.  Realis ---------');
for ja1:=1 to jat do
  for ja2:=1 to jat do
    for jb1:=1 to jbt do
      for jb2:=1 to jbt do begin
        // Compute one realisation
        vRealis:= avFF[1,1,ja1,jb1] + avFF[1,2,ja1,jb2] +
                  avFF[2,1,ja2,jb1] + avFF[2,2,ja2,jb2];

        DBW(1, ITS5(ja1) + ITS5(ja2) + ITS5(jb1) + ITS5(jb2) +
               FTS(vMean) + FTS(vRealis));
        end;


end;  // TMyWindow.Test1

procedure TMyWindow.Test2;
//========================
// 3 Settings, 2 possible results.
var
  iat,ibt,jat,jbt : integer;
  ia,ib,ja,jb     : integer;
  ia1,ib1         : integer;
  fStopLoop       : boolean;
  anA,anB         : array[1..5] of integer; // Possible experiment results
  avThetaA,
  avThetaB    : array[1..5] of Float;
  avecSetA,
  avecSetB    : array[1..5] of RVectR3Float;
  vSign,
  vNoise,
  vLambda,
  vMean,
  vRealis     // One realisation
              : Float;
begin
iat:=3; // Number of settings
ibt:=3;
jat:=2;
jbt:=2;

// Defining the settings
avThetaA[1] := 0.0;
aVThetaA[2] :=   pi/3;
aVThetaA[3] := 2*pi/3;
avThetaB[1] := pi-2*pi/6;
aVThetaB[2] := pi;
avThetaB[3] := pi+2*pi/6;

for ia:=1 to iat do
   avecSetA[ia]:=V3Assign(0.0, cos(avThetaA[ia]), sin(avThetaA[ia]));

for ib:=1 to ibt do
   avecSetB[ib]:=V3Assign(0.0, cos(avThetaB[ib]), sin(avThetaB[ib]));

// Noise in the entangled state
vNoise:=0.15;
vLambda:=1.0 - vNoise;

// Defining the Probabilities
DBW(1, '--- Pr --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         if odd(ja+jb) then vSign:= 1
         else               vSign:=-1;
         avPr[ia,ib,ja,jb]:=0.25*(vLambda*(1.0 - vSign * avecSetA[ia]*avecSetB[ib]) + vNoise);

         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avPr[ia,ib,ja,jb]));
         end;

// Marginal probabilities on Alice side
DBW(1, '--- P1 --------');
for ia:=1 to iat do
    for ja:=1 to jat do begin
      avP1[ia,ja]:=0;
      for jb:=1 to jbt do avP1[ia,ja]:=avP1[ia,ja] + avPr[ia,1,ja,jb];

//      DBW(1, ITS5(ia) + ITS5(ja) + FTS(avP1[ia,ja]));
      end;

// Marginal probabilities on Bob side
DBW(1, '--- P2 --------');
for ib:=1 to ibt do
    for jb:=1 to jbt do begin
      avP2[ib,jb]:=0;
      for ja:=1 to jat do avP2[ib,jb]:=avP2[ib,jb] + avPr[1,ib,ja,jb];

//      DBW(1, ITS5(ib) + ITS5(jb) + FTS(avP2[ib,jb]));
      end;

// Try to define the function of the random variables
DBW(1, '--- FF --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
//         avFF[ia,ib,ja,jb]:=(avPr[ia,ib,ja,jb] - avP1[ia,ja] * avP2[ib,jb]);
         avFF[ia,ib,ja,jb]:=avPr[ia,ib,ja,jb];

         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avFF[ia,ib,ja,jb]));
         end;

// Esperance of the combination of the random variables
vMean:=0;
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         vMean:=vMean + avFF[ia,ib,ja,jb] * avPr[ia,ib,ja,jb];
         end;

// Comparaison between the Esperance and the realisations.
DBW(1, '-- Mean  v.s.  Realis ---------');
for ia:=1 to iat do anA[ia]:=1;
for ib:=1 to ibt do anB[ib]:=1;

ib1:=1;
repeat
   // Compute one realisation
   vRealis:=0;
   for ia:=1 to iat do
      for ib:=1 to ibt do
         vRealis:=vRealis + avFF[ia,ib,anA[ia],anB[ib]];

   DBW(1, FTS(vMean) + FTS(vRealis) + FTS((vMean - vRealis)/vMean));

   // Next possible realisation
   ia1:=1;
   repeat
      fStopLoop:=TRUE;
      inc(anA[ia1]);
      if (anA[ia1] > jat) then begin
         anA[ia1]:=1;
         inc(ia1);
         fStopLoop:=FALSE;
         end;
      until fStopLoop or (ia1 > iat);

   if (ia1 > iat) then begin
      ib1:=1;
      repeat
         fStopLoop:=TRUE;
         inc(anB[ib1]);
         if (anB[ib1] > jbt) then begin
            anB[ib1]:=1;
            inc(ib1);
            fStopLoop:=FALSE;
            end;
         until fStopLoop or (ib1 > ibt);
      end;

   until (ib1 > ibt);


end;  // TMyWindow.Test2

function FuncF2(pData:pointer; var avX:array of Float):Float;
//=============================================================
// - Min of Sum(P(ia,ib,ja,jb) - delta(A(ia),ja)*delta(B(ib),jb)) * avV[ia,ib,ja,jb]
var
  ia,ib,ja,jb     : integer;
  ia1,ib1         : integer;
  jj              : integer;
  fStopLoop       : boolean;
  anA,anB         : array[1..5] of integer; // Possible experiment results
  vSum,
  vSumExp,
  vNorme,
  vMin            : Float;

begin
with PMyWindow(pData)^ do begin
vSum:=0;
// To do a huge sum, over iat*ibt indexes, each from 1 to jat  or  1 to jbt
for ia:=1 to iatt do anA[ia]:=1;
for ib:=1 to ibtt do anB[ib]:=1;

// Normalise the vector avX[jj]
vNorme:=0;
for jj:=1 to iatt*ibtt*jatt*jbtt do vNorme:=vNorme + sqr(avX[jj]);
vNorme:=sqrt(vNorme);
for jj:=1 to iatt*ibtt*jatt*jbtt do avX[jj]:=avX[jj] / vNorme;

vSumExp:=0;
vMin:=100000;
ib1:=1;
repeat
   // Compute one sum i.e. one product, matrix line times vector avX
   vSum:=0;
   jj:=0;
   for ia:=1 to iatt do for ib:=1 to ibtt do
   for ja:=1 to jatt do for jb:=1 to jbtt do begin
      inc(jj);
      if (anA[ia] = ja) and (anB[ib] = jb) then
           vSum:=vSum + (avPr[ia,ib,ja,jb] - 1) * avX[jj]
      else vSum:=vSum + (avPr[ia,ib,ja,jb]    ) * avX[jj];
      end;

   if (vMin > vSum) then vMin:=vSum;
   vSumExp:=vSumExp + exp(-50*vSum);

   // Next possible realisation, i.e. next line
   ia1:=1;
   repeat
      fStopLoop:=TRUE;
      inc(anA[ia1]);
      if (anA[ia1] > jatt) then begin
         anA[ia1]:=1;
         inc(ia1);
         fStopLoop:=FALSE;
         end;
      until fStopLoop or (ia1 > iatt);

   if (ia1 > iatt) then begin
      ib1:=1;
      repeat
         fStopLoop:=TRUE;
         inc(anB[ib1]);
         if (anB[ib1] > jbtt) then begin
            anB[ib1]:=1;
            inc(ib1);
            fStopLoop:=FALSE;
            end;
         until fStopLoop or (ib1 > ibtt);
      end;

   until (ib1 > ibtt);

vMinFunc:=vMin;
end; // with

FuncF2:=vSumExp;
DBW(128, 'funcF2=' + FTS(FuncF2) + FTS(vMin));
end;  // FuncF2

procedure TMyWindow.Test3;
//========================
// 2 Settings, 3 possible results, with detector efficiency
var
  iat,ibt,jat,jbt : integer;
  ia,ib,ja,jb     : integer;
  ia1,ib1         : integer;
  jj              : integer;
  fStopLoop       : boolean;
  anA,anB         : array[1..5] of integer; // Possible experiment results
  avThetaA,
  avThetaB    : array[1..5] of Float;
  avecSetA,
  avecSetB    : array[1..5] of RVectR3Float;
  vSign,
  vNoise,
  vLambda,
  vMean,
  vRealis     // One realisation
              : Float;
  vEffic      : Float;  // detector efficiency
  avEff       : array[1..3] of Float; // output probabilities

  oMinS       : TMinRNSearchRN;

begin
iat:=3; // Number of settings
ibt:=3;
jat:=3;
jbt:=3;

// Defining the settings
if (iat = 2) then begin
   avThetaA[1] := 0.0;
   aVThetaA[2] := pi/2;
   end;
if (ibt = 2) then begin
   avThetaB[1] := pi/4;
   aVThetaB[2] := 3*pi/4;
   end;

if (iat = 3) then begin
   avThetaA[1] := 0.0;
   aVThetaA[2] :=   pi/3;
   aVThetaA[3] := 2*pi/3;
   end;
if (ibt = 3) then begin
   avThetaB[1] := pi-2*pi/6;
   aVThetaB[2] := pi;
   avThetaB[3] := pi+2*pi/6;
   end;

for ia:=1 to iat do
   avecSetA[ia]:=V3Assign(0.0, cos(avThetaA[ia]), sin(avThetaA[ia]));

for ib:=1 to ibt do
   avecSetB[ib]:=V3Assign(0.0, cos(avThetaB[ib]), sin(avThetaB[ib]));

// Noise in the entangled state
//vNoise:=1-1/sqrt(2); // = Limit = 0.292893219
vNoise:=0.0;
vLambda:=1.0 - vNoise;

// Detector efficiency
// limit from the theory = 2/(1+sqrt(2)) = 0.82842712474619
vEffic:=0.9;

// result = 1 or 2 means that we got a click on exactly one detector
// result = 3      means that the detector didn't click
avEff[1]:=vEffic/2;
avEff[2]:=vEffic/2;
avEff[3]:=1.0 - vEffic;

// Defining the Probabilities
DBW(1, '--- Pr --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         if (ja < 3) and (jb < 3) then begin
            if odd(ja+jb) then vSign:= 1
            else               vSign:=-1;
            avPr[ia,ib,ja,jb]:=avEff[ja]*avEff[jb]*(vLambda*(1.0 - vSign * avecSetA[ia]*avecSetB[ib]) + vNoise);
            end
         else avPr[ia,ib,ja,jb]:=avEff[ja]*avEff[jb];
         DBW(8, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avPr[ia,ib,ja,jb]));
         end;

iatt:=iat;  ibtt:=ibt;  jatt:=jat;  jbtt:=jbt;

oMinS.Init(iat*ibt*jat*jbt);

// Data which must be given, before minimisation can start :
oMinS.nDim:=iat*ibt*jat*jbt;
oMinS.func:=@FuncF2;

// Initial vector
jj:=0;
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         inc(jj);
         oMinS.pvecXMin^[jj]:=1.0;
         end;

oMinS.vSizeEdge:=0.05;  // Starting search movements

// This can be changed. Here default values are shown.
oMinS.vTolRel   :=1e-3;
oMinS.vTolAbs   :=1e-3;
oMinS.nNbFuncEvalMax:=4000;
oMinS.pDataFunc :=@Self;

// Do the minimisation
oMinS.MinimisePowell;

// Result of the minimisation
DBW(1, 'Number of function evaluation : ' + ITS5(oMinS.nNbFuncEval));
DBW(1, 'Function minimum value :' + FTS(oMinS.vFXMin));
DBW(1, 'scalar product Minimum value :' + FTS(vMinFunc));

oMinS.Done;
end;  // TMyWindow.Test3

procedure TMyWindow.SearchX;
//==========================
// Search a vector  X  such that  avecV[ii]*X < 0 for all X
var
  nCpt,
  ii,jj       : integer;
  vNorme,
  vNormeX,
  vNormeV,
  vScalProd,
  vNormeMax,
  vNormeMin,
  vMin,
  vSum        : Float;
begin
// Normalise all vectors avecV[ii]
for ii:=1 to nNbLine do begin
   vNorme:=0;
   for jj:=1 to nNbCol do vNorme:=vNorme + sqr(avecV[ii][jj]);

   vNorme:=sqrt(vNorme);
   for jj:=1 to nNbCol do avecV[ii][jj]:=avecV[ii][jj] / vNorme;
   end;

// Loop to seach the vector X
nCpt:=0;
vNormeMax:=0;
vNormeMin:=1000;
repeat
   // vecX = sum of the vectors.
   vNormeX:=1e-8;
   for jj:=1 to nNbCol do begin
      vSum:=0;
      for ii:=1 to nNbLine do vSum:=vSum + avecV[ii][jj];

      vecX[jj]:=vSum;
      vNormeX  :=vNormeX + sqr(vSum);
      end;

   // Normalise the X vector
   vNormeX:=sqrt(vNormeX);
   for jj:=1 to nNbCol do vecX[jj]:=vecX[jj] / vNormeX;

   // Give more importance to vectors fare away from X
   vMin:=1.0;
   for ii:=1 to nNbLine do begin
      vScalProd:=0;
      vNormeV:=1e-12;
      for jj:=1 to nNbCol do begin
         vScalProd:=vScalProd + avecV[ii][jj] * vecX[jj];
         vNormeV  :=vNormeV + sqr(avecV[ii][jj]);
         end;

      // memorise the smallest scalar product. If it's >0 then finished.
      vNormeV:=sqrt(vNormeV);
      if (vNormeMax < vNormeV) then vNormeMax:=vNormeV;
      if (vNormeMin > vNormeV) then vNormeMin:=vNormeV;
      vScalProd:=vScalProd / vNormeV;
      if (vMin > vScalProd) then vMin:=vScalProd;

      // Change the vectors length
      for jj:=1 to nNbCol do avecV[ii][jj]:=avecV[ii][jj] * (1.0 - 0.75*vScalProd);
      end; // for ii:=1 to nNbLine do begin

   inc(nCpt);
   until (nCpt > 800) or (vMin > 0) or (vNormeMax > 100000.0);

// if vMin > 0, we found the wanted vector X
DBW(1, 'vMin=' + FTS(vMin) + FTS(vNormeMax) + FTS(vNormeMin) + ITS5(nCpt));

end;  // TMyWindow.SearchX

procedure TMyWindow.Test4;
//========================
// 2 Settings, 3 possible results, with detector efficiency
var
  iat,ibt,jat,jbt : integer;
  ia,ib,ja,jb     : integer;
  ia1,ib1         : integer;
  ii,jj           : integer;
  fStopLoop       : boolean;
  anA,anB         : array[1..5] of integer; // Possible experiment results
  avThetaA,
  avThetaB    : array[1..5] of Float;
  avecSetA,
  avecSetB    : array[1..5] of RVectR3Float;
  vSign,
  vNoise,
  vLambda,
  vMean
              : Float;
  vEffic      : Float;  // detector efficiency
  avEff       : array[1..3] of Float; // output probabilities
  strS        : ShortString;

begin
iat:=2; // Number of settings
ibt:=2;
jat:=3;
jbt:=3;

// Defining the settings
avThetaA[1] := 0.0;
aVThetaA[2] := pi/2;
avThetaB[1] := pi/4;
aVThetaB[2] := 3*pi/4;

for ia:=1 to iat do
   avecSetA[ia]:=V3Assign(0.0, cos(avThetaA[ia]), sin(avThetaA[ia]));

for ib:=1 to ibt do
   avecSetB[ib]:=V3Assign(0.0, cos(avThetaB[ib]), sin(avThetaB[ib]));

// Noise in the entangled state
// Theoretical limit = 1 - 1/sqrt(2) = 0.2928932
vNoise:=0.0;
vLambda:=1.0 - vNoise;

// Detector efficiency
// limit from the theory = 2/(1+sqrt(2)) = 0.82842712474619
vEffic:=0.84;

// result = 1 or 2 means that we got a click on exactly one detector
// result = 3      means that the detector didn't click
avEff[1]:=vEffic/2;
avEff[2]:=vEffic/2;
avEff[3]:=1.0 - vEffic;

// Defining the Probabilities
DBW(1, '--- Pr --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         if (ja < 3) and (jb < 3) then begin
            if odd(ja+jb) then vSign:= 1
            else               vSign:=-1;
            avPr[ia,ib,ja,jb]:=avEff[ja]*avEff[jb]*(vLambda*(1.0 - vSign * avecSetA[ia]*avecSetB[ib]) + vNoise);
            end
         else avPr[ia,ib,ja,jb]:=avEff[ja]*avEff[jb];
         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) + FTS(avPr[ia,ib,ja,jb]));
         end;

// Marginal probabilities on Alice side
DBW(1, '--- P1 --------');
for ia:=1 to iat do
    for ja:=1 to jat do begin
      avP1[ia,ja]:=0;
      for jb:=1 to jbt do avP1[ia,ja]:=avP1[ia,ja] + avPr[ia,1,ja,jb];

//      DBW(1, ITS5(ia) + ITS5(ja) + FTS(avP1[ia,ja]));
      end;

// Marginal probabilities on Bob side
DBW(1, '--- P2 --------');
for ib:=1 to ibt do
    for jb:=1 to jbt do begin
      avP2[ib,jb]:=0;
      for ja:=1 to jat do avP2[ib,jb]:=avP2[ib,jb] + avPr[1,ib,ja,jb];

//      DBW(1, ITS5(ib) + ITS5(jb) + FTS(avP2[ib,jb]));
      end;

// Try to define the function of the random variables
DBW(32, '--- FF --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
//         avFF[ia,ib,ja,jb]:=avPr[ia,ib,ja,jb] - avP1[ia,ja] * avP2[ib,jb];
         avFF[ia,ib,ja,jb]:=avPr[ia,ib,ja,jb];

// DBW(32, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +  FTS(avFF[ia,ib,ja,jb]));
         end;

// Esperance of the combination of the random variables
vMean:=0;
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         vMean:=vMean + avFF[ia,ib,ja,jb] * avPr[ia,ib,ja,jb];
         end;

// Computes the vectors which scalar product with a searched vector X must all be negative
for ia:=1 to iat do anA[ia]:=1;
for ib:=1 to ibt do anB[ib]:=1;

ib1:=1;
ii:=0;
repeat
   // Fill one vector
   inc(ii);
   jj:=0;
   strS:='';
   for ia:=1 to iat do
      for ib:=1 to ibt do
         for ja:=1 to jat do
            for jb:=1 to jbt do begin
               inc(jj);
               if (anA[ia] = ja) and (anB[ib] = jb) then
                    avecV[ii][jj]:= avPr[ia,ib,ja,jb] - 1.0
               else avecV[ii][jj]:= avPr[ia,ib,ja,jb];

               if (anA[ia] = ja) and (anB[ib] = jb) then
                    strS:=strS + '1 '
               else strS:=strS + '0 ';
               end;

   // Next vector
//   DBW(1, IntToStr(ii,2) + ' ' + strS);
   ia1:=1;
   repeat
      fStopLoop:=TRUE;
      inc(anA[ia1]);
      if (anA[ia1] > jat) then begin
         anA[ia1]:=1;
         inc(ia1);
         fStopLoop:=FALSE;
         end;
      until fStopLoop or (ia1 > iat);

   if (ia1 > iat) then begin
      ib1:=1;
      repeat
         fStopLoop:=TRUE;
         inc(anB[ib1]);
         if (anB[ib1] > jbt) then begin
            anB[ib1]:=1;
            inc(ib1);
            fStopLoop:=FALSE;
            end;
         until fStopLoop or (ib1 > ibt);
      end;

   until (ib1 > ibt);

iatt:=iat;  ibtt:=ibt;  jatt:=jat;  jbtt:=jbt;

nNbLine:=ii;
nNbCol :=jj;

// Search a vector  X  such that  avecV[ii]*X > 0 for all X
SearchX;

end;  // TMyWindow.Test4

function TMyWindow.WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT;
//=============================================================
// Handles the menu messages
Var wNrMenu     : word;
begin
WMCommand := 0; // Default return value when processing a message

if (lPar <> 0) then begin
   // call the parent WMCommand method.
   WMCommand:=inherited WMCommand(wPar, lPar);
   exit;
   end;

// for the menus,  lPar = 0
//=========================
wNrMenu   := LoWord(wPar);
Case wNrMenu Of
   101 : Test1;
   102 : Test2;
   103 : Test3;
   104 : Test4;

   else WMCommand:=inherited WMCommand(wPar, lPar);
   end;  // Case NrMenu Of
end;  // TMyWindow.WMCommand

function TMyWindow.WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT;
//======================================================================================
begin
// Default treatment of a message. Does nearly nothing
WindowProc := inherited WindowProc(hWindow, wMsg, wPar, lPar);
end;  // TMyWindow.WindowProc

Begin // MAIN ******
// Name under which the main window is registered. Spaces are not allowed
oApp.Init('tzNonLocal01');

// Creation of the main window
new(poMyMainWin, Init('tzNonLocal01, to characterise non locality'));

oApp.Run;
oApp.Done;  // The main window object is disposed here.

glf:=glf; // To avoid a Hint
End.

