unit wSurface2DFrame0;

{ Feel free to use this unit as you want.
  First developped by Bernard Gisin
  This unit uses object programming to simplify the developpment
  of programs.

  This unit helps drawing surfaces, defined by funtions
  F : RxR -> R

  procedure Surf2DCompute(var wErrorV:word32);
  //==========================================
  // This is the procedure to call to draw a function.
  // see tEx055.pp

  procedure Visualise(wPar:WPARAM);
  //===============================
  // To rotate around the surface
  // It is typically called when a WM_KEYDOWN message arrives, with
  // wPar in [VK_NUMPAD0 .. VK_DIVIDE]

  Remark :
     It is an old program I wrote in 1984 and modified in 1992.
  I changed some little details here, but let many others unchanged.

  Optimisation :
  """"""""""""""
  Most of the time is spend in the drawing of the surface, thus
  optimising the projection will not increase speed.

  Mathematics and definitions :
  """"""""""""""""""""""""""""""
  A surface is given through :
  vSurfXOrg, vSurfYOrg, vSurfDX, vSurfDY,
  Mat[nMatMinIndexX..nMatMaxIndexX, nMatMinIndexY..nMatMaxIndexY]
  defines the points in the space :
  (vSurfXOrg + nX * vSurfDX, vSurfXOrg + nX * vSurfDX,    Mat[ ~nX, ~nY])
  nX:=nUseMinIndexX..nUseMaxIndexX, nY:=nUseMinIndexY..nUseMaxIndexY

  We want to project this surface on a plan corresponding to the screen
  The plan is defined by :
  (0, 0, 0) = the origine of the plan = center of the screen
  P = (vP1, vP2, vP3), is normal to the plan.

  The projection is paralelle to P

  Basis of the plan :
  '''''''''''''''''''
  E1 = (vE1x, vE1y, 0) is the intersection of the plan with the ground z=0
  If there is no intersection, then E1=Q + (1,0,0)

  E1 = (-vP2, vP1, 0) normalised

  E2 = P x E1  = the vector product.
  Thus (E1, E2, P) is an orthogonal basis of the space.

  Plan = [ u*E1 + v*E2 | u,v in Real ]

  (u,v) will be the coordinate on the screen of the projected point.

  If V = (vX, vY, vZ) is a point of the surface, the intersection of the line
  V-P with the plan will give the coordinate (u, v) on the screen

  Line = [ P + w * V-P | w in Real ]

  Thus we have to solve :
  P + w * V-P = u*E1 + v*E2  to compute  (u, v)

  Easier is to notice that :  (E1 and E2 normalised)
  u = E1 * V    scalar product.
  v = E2 * V
----------------------------------}

// To tell FreePascal that the application is a Graphical User Interface
// i.e. is running in windows, not in text mode.
{$APPTYPE GUI}

INTERFACE
//"""""""

uses wControl0, wWindowBase0, wGlobals2, windows;

type
  MatrixIn  = array[0..200000000] of Float;
  PMatrixIn = ^MatrixIn; // Only the pointer will be used.

// Possible errors
const
  errSurf2D_MemoryOverflow             =    1;
  errSurf2D_BadDimensions              =    2;
  errSurf2D_WindowTooSmall             =    3;

// Drawing options
const
  flaSurf2D_UserLimits                 =    1;
  // Scaling are fixed in  vXProjMin, vYProjMin, vXProjMax, vYProjMax
  // Do not recompute them.
  // A normal use it to call  Surf2DCompute  without this flag, then
  // turn this flag on, and call  Visualise(wPar)
  // when a WM_KEYDOWN message arrives.

  flaSurf2D_DrawGroundFrame            =    2;
  flaSurf2D_DrawAxes                   =    4;

const
  nXPixelMax=4095;
  // = max number of pixels in the X and Y axis
  // We suppose that there are not more that 8000 pixels in a line or
  // in a column

type
  place=(inferior,hidden,superior);
  ivector_limit=array[0..nXPixelMax] of integer;

  SingleType = FloatS;
  TArrayMatrixFloatsS = array[0..400000000] of SingleType; // only pointer on it are used
  TArrayMatrixInteger = array[0..400000000] of integer; // only pointer on it are used
  TArrayVectorInteger = array[0..400000000] of integer; // only pointer on it are used
  PArrayMatrixFloatsS = ^TArrayMatrixFloatsS;
  PArrayMatrixInteger = ^TArrayMatrixInteger;
  PArrayVectorInteger = ^TArrayVectorInteger;

type
  PSurface2DFrame0 = ^TSurface2DFrame0;
  TSurface2DFrame0 = object(TControl)
  //"""""""""""""""""""""""""""""""""
  // Handle on the window. (=0 when not created)
//  hWin     : HWND;  // Handle on the window, got from TWindowBase
//  hParent : HWND;  // Handle on the parent, got from TWindowBase
//  poParent: PWindowBase0;  // Pointer on the parent, got from TWindowBase

  // These values MUST be defined before calling  Surf2DCompute
  //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
  pavMat   : PMatrixIn;
  // (vSurfXOrg + nX * vSurfDX,  vSurfXOrg + nX * vSurfDX,  pavMat^[nX*nMDY + nY])
  // with  nX:=0..nXDim,  nY:=0..nYDim
  // define the surface

  nMatMinIndexX,
  nMatMaxIndexX,
  nMatMinIndexY,
  nMatMaxIndexY : int32;
  // Tells which are the real indexes and dimensions of the matrix.
  // This means that the matrix has been defined as an
  // array[nMatMinIndexX..nMatMaxIndexX, nMatMinIndexY..nMatMaxIndexY] of Float;

  nDefinedMinIndexX,
  nDefinedMaxIndexX,
  nDefinedMinIndexY,
  nDefinedMaxIndexY : int32;
  // Tells which indexes have been assigned to the matrix.
  // if  Mat  is the matrix defined with the above type, then
  // Mat[nx,ny] has been defined for
  // nx in [nDefinedMinIndexX..nDefinedMaxIndexX]
  // ny in [nDefinedMinIndexY..nDefinedMaxIndexY]
  // For other values of nx and ny, the matrix could not have been assigned values.

  // These values can be defined.
  //"""""""""""""""""""""""""""""
  nUseMinIndexX,
  nUseMaxIndexX,
  nUseMinIndexY,
  nUseMaxIndexY,
  nDeltaIndexX,nDeltaIndexY  : int32;
  // Tells which indexes you want to use to show the surface.
  // if  Mat  is the matrix defined with the above type, then
  // the used values of the matrix will be
  // Mat[nUseMinIndexX + nx*nDeltaIndexX, nUseMinIndexY + ny*nDeltaIndexY]
  // nx:=0..(nUseMaxIndexX - nUseMinIndexX) div nDeltaIndexX
  // ny:=0..(nUseMaxIndexY - nUseMinIndexY) div nDeltaIndexY

  vP1,vP2,vP3      : Float;
  // Projection point coordonate

  // Given by the user : (See comments at the beginning or just below)
  // Sizes in the same units as the  Z  axis of the  X  and  Y  axis
  vSurfXOrg, vSurfYOrg, // (X, Y) position of the point  Mat[0,0]
  vSurfDX, vSurfDY : Float;

  vZMinLimit,      // Cut all matrix values below this limit
  vZMaxLimit       // Cut all matrix values above this limit
                   : Float;

  wFlags           : word32;
  // Some drawing flags, see the constant  flaSurf2D_...

  ColorSuperior,
  ColorInferior,
  ColorFrame,
  ColorAxes        : COLORREF;
  // some colors

  vProjZoomFactor : Float;
  // Set to one by default, means that the projection will take all
  // the window place.
  // Later, if the image is moved, then some part may be out of the window.
  // Thus you can define a new Zoom factor.
  // For example, 0.8 will have a little smaller image

  nXProjTranslate,
  nYProjTranslate : int32;
  // Set to zero by default, but can translate the image on the screen

  vXProjMin,
  vYProjMin,
  vXProjMax,
  vYProjMax : Float;
  // You can fixe the projections dimensions.
  // Usually, you call once  Surf2DCompute  to let them be computed, them
  // you add the flag flaSurf2D_UserLimits
  // (wFlags:=wFlags or flaSurf2D_UserLimits;)
  // Then the dimensions are fixed and you can rotate around the surface.
  // by zooming, it will have the same effect as changing these values.
  // These values are rarely changed by the user.

   vZFuncMin,
   vZFuncMax : Float;
   // Minimal & maximal value of the function.
   // It is computed automatically if the flag  flaSurf2D_UserLimits
   // is not set.
   // When the flag is set, it is used to fixe min & max values
   // of the function, and thus fixe the center of the cube containing
   // the function.

  // Internal data, used to compute and draw the projection of the surface.
  //'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  nMDY : int32;
  // Abreviation of wMatrixDimentionY
  // This is the real dimension of the second index of the matrix
  // nMDY = 1 + nMatMaxIndexY - nMatMinIndexY;

  hDCDraw : HDC;
  // Device context of the window, where the surface is drawn

  hPenSuperior,    // color of the superior face
  hPenInferior,    // color of the inferior face
  hPenFrame,       // color on the frame
  hPenAxes,        // color of the X - Y axis
  hPenOld          // original pen of the device context.
                   : HPEN;

   vZMin,vZMax     : Float;
   // Min and max values of the matrix.

  nXTranslate,
  nYTranslate : integer;
  // Translation on the plan, to center the image on the screen.

  vTheta,
  vPhi             : Float;
  // To rotate around the surface.
  // This modifies the projection point.

private
   position : place;

   i1,j1,i2,j2 : integer;
      ymin,ymax,ymin_new,ymax_new : ivector_limit; { upper and lower limits }
      w1,w2,w3,ww1,ww2,u1,u2,u3,v1,v2,v3 : SingleType;
      iline : integer;
      icas : integer;
      ixleft,ixmed,ixright : integer; {ixmed=(ixleft+ixright)/2}
      iydown,iymed,iyup : integer;    {iymed=(iydown+iyup)/2}
      LastPosition : place; { mmorise la dernire position }
      nXNbDX,nYNbDY   : integer; { = number of interval in X and Y wanted }

      XX1       : integer;
      YY1       : integer;
      VX1       : integer;
      VY1       : integer;
      VX2       : integer;
      VY2       : integer;

   panXLine,panYLine : PArrayVectorInteger; {used with PolylLine }

   panX, panY : PArrayMatrixInteger;
   // integer matrix are used to accelerate the calculus

  pavX,pavY : PArrayMatrixFloatsS; // X and Y projexction of the function stored in pavMat

  wSizeVectInt,   // number of bytes allocated for the panXLine and panYLine
  wSizeMatFloatS, // number of bytes allocated for the pavX and pavY matrices
  wSizeMatInt     // number of bytes allocated for the panX and panY matrices
                  : word32;
  nMDFY, // Matrix Dimention Float Y dimension, i.e. for the second index
  nMDIY  // Matrix Dimention Integer Y dimension, i.e. for the second index
         : int32;
  // pavX^[wX*nMDFY + wY] = the X coordinate projection of the (nx, ny) point.

  vXMin,vXMax,
  vYMin,vYMax
              : SingleType;
public

  constructor Init(poParentW:PWindowBase0);
  destructor  Done; virtual;

  procedure CtrlCreate; virtual;
  // Create the window of the control.
  // Must have a parent object window, if not, nothing happens.

  procedure MultiLine(nLineP:integer);

  procedure PutPoint(ix1, jy1, ix2, jy2:integer);

  procedure draw_line;

  procedure project(x,y,z:Float; var u,v:SingleType);

  procedure projectInt(x,y,z:Float; var nu,nv:integer);

  procedure TestInputValidity(var wErrorV:word32);
  // Test the validity of the inputs

  procedure ComputProjectionMatrix;
  // Determination of the orthogonal projection on the plane p1*x+vP2*y+vP3*z=0

  procedure SurfaceProjection;
  // Projection of the surface on the plane and computation of the limits

  procedure ConvertToInteger;
  // Transformation of the coord. in integer
  // between (ixleft,ixright) for the x-axis,
  // between (iydown,iyup   ) for the y-axis.
  // such that the proportions are respected

  procedure DrawGroundFrameAndAxes;
  // Drawing of the ground frame

  procedure DrawSurface;
  // Drawing of the surface

  procedure Initialise(wOptions:word32);
  //====================================
  // Should be called if a new surface or a new projection point has been
  // defined.
  // If wOptions = 1 then default values are initialised.
  // In any case, compute the angle  vTheta and vPhi from the projection point.

  procedure Surf2DCompute(var wErrorV:word32);
  //==========================================
  // This is the procedure to call to draw a function.
  // see tEx055.pp

  procedure Visualise(wPar:WPARAM);
  //===============================
  // To rotate around the surface
  // It is typically called when a WM_KEYDOWN message arrives, with
  // wPar in [VK_NUMPAD0 .. VK_DIVIDE]

  procedure EraseBackground;
  //========================
  // Erase the background of the window

  procedure WMPaint; virtual;
  // Paint the popup window

  function CtrlWindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  // Handles the window messages
  // A SendMessage(hWin, ...) must not be called from here,
  // to avoid calling loops. (SendMessage(hWin...) calls this function

  end; // TSurface2DFrame0 = object(TWindowBase)
  TSurface2DFrame =  TSurface2DFrame0;
  PSurface2DFrame = ^TSurface2DFrame;

IMPLEMENTATION
//""""""""""""

uses wUtiles0, wDebug0;

//***************************************************
// TSurface2DFrame0 = object(TControl)
//***************************************************

constructor TSurface2DFrame0.Init(poParentW:PWindowBase0);
//=========================================================
begin
inherited Init(poParentW);

wWindowType:=GGLCTE_CTRL_Surf2DFrame;

// These values MUST be defined before calling  Surf2DCompute
//""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
nMatMinIndexX := 32767; // The matrix has not been defined yet
nMatMaxIndexX :=-32767;
nMatMinIndexY := 32767;
nMatMaxIndexY :=-32767;

nDefinedMinIndexX := 32767;
nDefinedMaxIndexX :=-32767;
nDefinedMinIndexY := 32767;
nDefinedMaxIndexY :=-32767;

// These values can be defined.
//"""""""""""""""""""""""""""""
nUseMinIndexX := 32767;  // tells to give the same values as the nDefined...
nUseMaxIndexX :=-32767;
nUseMinIndexY := 32767;
nUseMaxIndexY :=-32767;

nDeltaIndexX  :=1; // no points are skipped, i.e. view with maximum resolution
nDeltaIndexY  :=1;

// Sizes in the same units as the  Z  axis of the  X  and  Y  axis
vSurfXOrg :=0;
vSurfYOrg :=0;
vSurfDX   :=1;
vSurfDY   :=1;

vP1 :=-1;
vP2 :=-2;
vP3 := 3;

vZMinLimit :=-1e30;
vZMaxLimit := 1e30;

wFlags  :=0;

ColorSuperior :=RGB(255,255,255); // white
ColorInferior :=RGB(255,  0,  0); // red
ColorFrame    :=RGB(  0,  0,255); // Blue
ColorAxes     :=RGB(  0,255,  0); // LightGreen

vProjZoomFactor:=1.0;
nXTranslate :=0;
nYTranslate :=0;

// Not common to fixe these values, but they can be fixed
vXProjMin := 0;
vYProjMin := 0;
vXProjMax := 0;
vYProjMax := 0;
vZFuncMin := 0;
vZFuncMax := 1;

// The whole window is used
vXSizeRel  := 1;
vYSizeRel  := 1;

dwStyle:=WS_Child or WS_VISIBLE;

//=======================================================
// Internal use
hPenSuperior :=0;
hPenInferior :=0;
hPenFrame    :=0;
hPenAxes     :=0;
hPenOld      :=0;

hDCDraw:=0;

vTheta :=0;
vPhi   :=0;

panXLine :=NIL;
panYLine :=NIL;

panX     :=NIL;
panY     :=NIL;

pavX     :=NIL;
pavY     :=NIL;

wSizeVectInt  :=0;
wSizeMatFloatS:=0;
wSizeMatInt   :=0;
end;  // TSurface2DFrame0.Init

destructor TSurface2DFrame0.Done;
//===============================
begin
if (hPenSuperior <> 0) then DeleteObject(hPenSuperior);
if (hPenInferior <> 0) then DeleteObject(hPenInferior);
if (hPenFrame <> 0)    then DeleteObject(hPenFrame);
if (hPenAxes <> 0)     then DeleteObject(hPenAxes);

// If memory has been allocated, desallocate it.
if (wSizeVectInt > 0) then begin
   Freemem(panXLine, wSizeVectInt);
   Freemem(panYLine, wSizeVectInt);
   wSizeVectInt:=0;
   end;

if (wSizeMatFloatS > 0) then begin
   Freemem(pavX, wSizeMatFloatS);
   Freemem(pavY, wSizeMatFloatS);
   wSizeMatFloatS:=0;
   end;

if (wSizeMatInt > 0) then begin
   Freemem(panX, wSizeMatInt);
   Freemem(panY, wSizeMatInt);
   wSizeMatInt:=0;
   end;

inherited Done;
end;  // TSurface2DFrame0.Done

procedure TSurface2DFrame0.CtrlCreate;
//====================================
Begin
strClassName:=gglstrClassWinName;
inherited CtrlCreate;
End;  // TSurface2DFrame0.CtrlCreate

procedure TSurface2DFrame0.MultiLine(nLineP:integer);
//===================================================
var   ii : integer;

begin
if position <> LastPosition then begin
   if position = superior then SelectObject(hDCDraw, hPenSuperior);
   if position = inferior then SelectObject(hDCDraw, hPenInferior);
   LastPosition:=position;
   end;

if position = inferior then begin
   MoveToEx(hDCDraw, panXLine^[1],panYLine^[1]+1, NIL);
   for ii:=2 to nLineP do LineTo(hDCDraw, panXLine^[ii], panYLine^[ii]+1);
   end
else if position = superior then begin
   MoveToEx(hDCDraw, panXLine^[1], panYLine^[1]-1, NIL);
   for ii:=2 to nLineP do LineTo(hDCDraw, panXLine^[ii], panYLine^[ii]-1);
   end
else begin
   MoveToEx(hDCDraw, panXLine^[1], panYLine^[1], NIL);
   for ii:=2 to nLineP do LineTo(hDCDraw, panXLine^[ii], panYLine^[ii]);
   end;
end;  // TSurface2DFrame0.MultiLine

procedure TSurface2DFrame0.PutPoint(ix1, jy1, ix2, jy2:integer);
//==============================================================
begin { - PutPoint - }
panXLine^[iline]:=ix1;
panYLine^[iline]:=jy1;
inc(iline);
panXLine^[iline]:=ix2;
panYLine^[iline]:=jy2;
end;  // TSurface2DFrame0.PutPoint

procedure TSurface2DFrame0.draw_line;
//===================================
var   i,ix1,ix2,ydep,ydes,deltax,deltay : integer;
begin
ix1 :=panX^[i1*nMDIY + j1];
ix2 :=panX^[i2*nMDIY + j2];
ydep:=panY^[i1*nMDIY + j1];
if ix1=ix2 then ix1:=ix1-1;
i:=ix1;
deltax:=ix2-ix1;
deltay:=panY^[i2*nMDIY + j2]-ydep;

repeat
  i:=i+1;
  ydes:=ydep + (deltay * (i-ix1)) div deltax;

    case position of
    superior : begin
               if ydes > ymin[i] then begin
                  { --- quit the superior part --- }
                  iline:=iline+1;
                  PutPoint(i-1,  {point precedent doit etre la et correct}
                           ydep + deltay * (i-1-ix1) div deltax,
                           i,
                           ymin[i]);  {point (faux) evite interrupt. segment}
                  MultiLine(iline);

                  { --- enter the hidden part or pass to the inferior part }
                  if ydes < ymax[i] then position:=hidden { hidden part }
                  else begin
                     { --- we pass from the superior to the inferior part }
                     iline:=1;
                     PutPoint(i,     {point (faux) evite interrupt. segment}
                              ymax[i]+1,
                              i,
                              ydes+1); {point correct pour depart de segment}

                     position:=inferior;
                     end;

                  end; {else stay in the superior part }
               end; { - begin }

    hidden : begin
             if ydes <= ymin[i] then begin
                { --- enter the superior part}
                iline:=1;
                PutPoint(i,     {point (faux) evite interrupt. segment}
                         ymin[i]-1,
                         i,
                         ydes-1); {point correct pour depart de segment}

                if ydes >= ymax[i] then begin
                   { --- si ymin[i] et ymax[i] n'ont pas encore t dfini }
                   {     cas rare, mais trs visible }
                   iline:=1;
                   PutPoint(i,
                            ydes-1,
                            i,
                            ydes-1); {point correct pour depart de segment}
                   end;

                position:=superior;
                end
             else if ydes >= ymax[i] then begin
                { --- enter the inferior part }
                iline:=1;
                PutPoint(i,     {point (faux) evite interrupt. segment}
                         ymax[i]+1,
                         i,
                         ydes+1); {point correct pour depart de segment}
                position:=inferior;
                end; { - if }
             end; { - begin }

    inferior : begin
               if ydes < ymax[i] then begin
                  { --- quit the inferior part --- }
                  iline:=iline+1;

                  PutPoint(i-1,  {point precedent doit etre la et correct}
                           ydep + deltay * (i-1-ix1) div deltax,
                           i,
                           ymax[i]);  {point (faux) evite interrupt. segment}

                  MultiLine(iline);

                  { --- enter the hidden part or pass to the superior part }
                  if ydes > ymin[i] then position:=hidden { hidden part }
                  else begin
                     { --- we pass from the inferior to the superior part }
                     iline:=1;
                     PutPoint(i,     {point (faux) evite interrupt. segment}
                              ymin[i]-1,
                              i,
                              ydes-1); {point correct pour depart de segment}

                     position:=superior;
                     end;

                  end; {else stay in the inferior part }
               end; { - begin }

    end; {case}

  if ymin_new[i]>ydes then ymin_new[i]:=ydes;
  if ymax_new[i]<ydes then ymax_new[i]:=ydes;
  until i>=ix2;

if position<>hidden then begin
   iline:=iline+1;
   panXLine^[iline]:=ix2;
   panYLine^[iline]:=ydes;
   end;
end;  // TSurface2DFrame0.draw_line

procedure TSurface2DFrame0.project(x,y,z:Float; var u,v:SingleType);
//==================================================================
begin
u:=x*u1+y*u2 {+z*u3 , but u3=0};
v:=x*v1+y*v2+z*v3;
end;  // TSurface2DFrame0.project

procedure TSurface2DFrame0.projectInt(x,y,z:Float; var nu,nv:integer);
//====================================================================
var   u, v : SingleType;
begin
u:=x*u1+y*u2 {+z*u3 , but u3=0};
v:=x*v1+y*v2+z*v3;

// The projection + translation is directly on the right place on the screen.
nu:=nXTranslate + Round(u);
nv:=nYTranslate + Round(v);
end;  // TSurface2DFrame0.projectInt

procedure TSurface2DFrame0.TestInputValidity(var wErrorV:word32);
//===============================================================
// Test the validity of the inputs
begin
if (nMatMinIndexX     >= nMatMaxIndexX) or
   (nMatMinIndexY     >= nMatMaxIndexY) or
   (nDefinedMinIndexX >= nDefinedMaxIndexX) or
   (nDefinedMinIndexY >= nDefinedMaxIndexY) then begin
   wErrorV:=errSurf2D_BadDimensions;
   exit;
   end;

// Number of indexes of the second index of the matrix
nMDY:=1 + nMatMaxIndexY - nMatMinIndexY;

if (nUseMinIndexX >= nUseMaxIndexX) then begin
   // Default values
  nUseMinIndexX :=nDefinedMinIndexX;
  nUseMaxIndexX :=nDefinedMaxIndexX;
  end;

if (nUseMinIndexY >= nUseMaxIndexY) then begin
   // Default values
  nUseMinIndexY :=nDefinedMinIndexY;
  nUseMaxIndexY :=nDefinedMaxIndexY;
  end;

// Test of input coherence
if (nUseMinIndexX < nDefinedMinIndexX) then nUseMinIndexX :=nDefinedMinIndexX;
if (nUseMaxIndexX > nDefinedMaxIndexX) then nUseMaxIndexX :=nDefinedMaxIndexX;
if (nUseMinIndexY < nDefinedMinIndexY) then nUseMinIndexY :=nDefinedMinIndexY;
if (nUseMaxIndexY > nDefinedMaxIndexY) then nUseMaxIndexY :=nDefinedMaxIndexY;

if (nDeltaIndexX > nUseMaxIndexX - nUseMinIndexX) then
    nDeltaIndexX:= nUseMaxIndexX - nUseMinIndexX;
if (nDeltaIndexX < 1) then nDeltaIndexX:=1;

if (nDeltaIndexY > nUseMaxIndexY - nUseMinIndexY) then
    nDeltaIndexY:= nUseMaxIndexY - nUseMinIndexY;
if (nDeltaIndexY < 1) then nDeltaIndexY:=1;

// number of intervals in the  X  direction
nXNbDX:=(nUseMaxIndexX - nUseMinIndexX) div nDeltaIndexX;

// number of intervals in the  Y  direction
nYNbDY:=(nUseMaxIndexY - nUseMinIndexY) div nDeltaIndexY;

wErrorV:=0;
end;  // TSurface2DFrame0.TestInputValidity

procedure TSurface2DFrame0.ComputProjectionMatrix;
//================================================
// Determination of the orthogonal projection on the plane p1*x+vP2*y+vP3*z=0
var
  p12norme,
  pnorme      : Float;
begin
// if vP1=vP2=vP3=0 then the standard value p=(-3,-2,2) is chosen
if (vP1=0) and (vP2=0) and (vP3=0) then begin vP1:=-3;  vP2:=-2;  vP3:=2; end;

p12norme:=sqr(vP1)+sqr(vP2);
pnorme:=sqrt(p12norme+sqr(vP3));
p12norme:=sqrt(p12norme);
{ -------- unitary vector colinear with p -------- }
w1:=vP1/pnorme;
w2:=vP2/pnorme;
w3:=vP3/pnorme;
{ -------- projection of  P  on the ground and normalisation ------- }
if p12norme=0 then begin
   ww1:=1;
   ww2:=0;
   end
else begin
   ww1:=vP1/p12norme;
   ww2:=vP2/p12norme;
   end;
{ ----- intersection of the plane with the ground -------- }
u1:=-ww2;
u2:= ww1;
u3:= 0;

// This solves a problem and gives many others
//if (u1 < 0) then begin u1:=-u1;  u2:=-u2;  end;

{ ------ v = w cross u ------- }
v1:=w2*u3-w3*u2;
v2:=w3*u1-w1*u3;
v3:=w1*u2-w2*u1;
{ ------ (Ui,Vi,Wi) is an orthonormal system, the projection will be : }
{ ------ u=x*u1+y*u2+z*u3 }
{ ------ v=x*v1+y*v2+z*u3 }

// Angle of projection
if (vTheta = 0) and (vPhi = 0) then begin
   vTheta:=arcsin(w3);
   if ( w1 <> 0) then vPhi  :=arcTan(w2 / w1)
   else vPhi:=Pi / 2.0;
   end;
end;  // TSurface2DFrame0.ComputProjectionMatrix

procedure TSurface2DFrame0.SurfaceProjection;
//===========================================
// Projection of the surface on the plane and computation of the limits
var
  ix,iy,
  ixtemp,iytemp,
  itemp,
  nXfDX,
  nYfDY
              : int32;
  x,y,z       : Float;
  u,v         : SingleType;

begin
{ ---- Determination how the data should be stored in order to draw    }
{ ---- the nearest part of the surface first.                          }
{ ---- It corrsepond to a rotation of the surface.                     }
if vP1<0 then begin
   if vP2<0 then icas:=00  {vP2<0 , vP1<0}
   else          icas:=10; {vP2>0 , vP1<0}
   end
else begin
   if vP2<0 then icas:=01  {vP2<0 , vP1>0}
   else          icas:=11; {vP2>0 , vP1>0}
   end;

if icas in [01,10] then begin
   // The X and Y coordinates are changed
   nMDIY:=1 + nDefinedMaxIndexX - nDefinedMinIndexX;
   nMDFY:=1 + nDefinedMaxIndexX - nDefinedMinIndexX;
   end;

{ ---------- projection ------------- }
vXMin:=1.0e30;
vXMax:=-vXMin;
vYMin:= vXMin;
vYMax:=-vYMin;
vZMin:= vXMin;
vZMax:=-vYMin;

for ix:=0 to nXNbDX do begin
    nXfDX:=nUseMinIndexX - nMatMinIndexX + ix * nDeltaIndexX;
    x:=vSurfXOrg + vSurfDX*(nXfDX + nMatMinIndexX);

    for iy:=0 to nYNbDY do begin
        nYfDY:=nUseMinIndexY - nMatMinIndexY + iy * nDeltaIndexY;
        y:=vSurfXOrg + vSurfDY*(nYfDY + nMatMinIndexY);
        z:=pavMat^[nXfDx*nMDY + nYfDY];

        if      (z < vZMinLimit) then z:=vZMinLimit
        else if (z > vZMaxLimit) then z:=vZMaxLimit;
        project(x,y,z,u,v);
        if u<vXMin then vXMin:=u;
        if u>vXMax then vXMax:=u;
        if v<vYMin then vYMin:=v;
        if v>vYMax then vYMax:=v;
        if (vZMin > z) then vZMin:=z;
        if (vZMax < z) then vZMax:=z;
        { -- memorise U and V -- }
          case icas of
          00: begin ixtemp:=ix;        iytemp:=iy;        end;
          01: begin ixtemp:=iy;        iytemp:=nXNbDX-ix; end;
          10: begin ixtemp:=nYNbDY-iy; iytemp:=ix;        end;
          11: begin ixtemp:=nXNbDX-ix; iytemp:=nYNbDY-iy; end;
          end; {case}
        pavX^[ixtemp*nMDFY + iytemp]:=u;
        pavY^[ixtemp*nMDFY + iytemp]:=v;
        end;
    end;

{ --- if the system has been rotated --- }
if icas in [01,10] then begin
   itemp:=nXNbDX;
   nXNbDX:=nYNbDY;
   nYNbDY:=itemp;
   end;
end;  // TSurface2DFrame0.SurfaceProjection

procedure TSurface2DFrame0.ConvertToInteger;
//==========================================
// Transformation of the coord. in integer
// between (ixleft,ixright) for the x-axis,  }
// between (iydown,iyup   ) for the y-axis.
// such that the proportions are respected
var
  ix,iy       : int32;
  vXCenter,
  vYCenter,
  vSizeX,
  vSizeY
              : SingleType;
begin
if ((wFlags and flaSurf2D_UserLimits) <> 0) then begin { frame limits are fixed }
   vXMin:=vXProjMin;
   vYMin:=vYProjMin;
   vZMin:=vZFuncMin;
   vXMax:=vXProjMax;
   vYMax:=vYProjMax;
   vZMax:=vZFuncMax;
   end
else begin { transmitting the frame limits to the user }
   vXProjMin:=vXMin;
   vYProjMin:=vYMin;
   vZFuncMin:=vZMin;
   vXProjMax:=vXMax;
   vYProjMax:=vYMax;
   vZFuncMax:=vZMax;
   end;

if vXMax=vXMin then vXMin:=vXMin-1.0e-3;
if vYMax=vYMin then vYMin:=vYMin-1.0e-3;
if (vZMin = vZMax) then begin vZMin:=vZMin - 1.0; vZMax:=vZMax + 0.5; end;

// The center of the cube containing the surface comes in the center of
// the screen.
project( vSurfXOrg + vSurfDX*(nUseMaxIndexX + nUseMinIndexX)/2.0,
         vSurfYOrg + vSurfDY*(nUseMaxIndexY + nUseMinIndexY)/2.0,
         (vZMin + vZMax) / 2, vXCenter, vYCenter);

vSizeX:=(ixRight - ixLeft) / (vXMax - vXMin);

if (vYMax - vYMin) * vSizeX > (iyUp - iyDown) then
   vSizeX:=(iyUp - iyDown) / ((vYMax - vYMin));

vSizeX:=vSizeX * vProjZoomFactor;
vSizeY:=-vSizeX;

// Modify the matrix projection such that it project directly in the
// screen coordinates.
u1 :=u1 * vSizeX;
u2 :=u2 * vSizeX;
v1 :=v1 * vSizeY;
v2 :=v2 * vSizeY;
v3 :=v3 * vSizeY;

nXTranslate :=ixmed - Round(vXCenter*vSizeX) + nXProjTranslate;
nYTranslate :=iymed - Round(vYCenter*vSizeY) + nYProjTranslate;

// Scale the projection to fit on the sceen.
for ix:=0 to nXNbDX do
    for iy:=0 to nYNbDY do begin
        panX^[ix*nMDFY + iy]:=Round(pavX^[ix*nMDFY + iy] * vSizeX) + nXTranslate;
        panY^[ix*nMDFY + iy]:=Round(pavY^[ix*nMDFY + iy] * vSizeY) + nYTranslate;
        end;
end;  // TSurface2DFrame0.ConvertToInteger

procedure TSurface2DFrame0.DrawGroundFrameAndAxes;
//================================================
// Drawing of the ground frame and the axis
const
  avXFact : array[1..6] of Float = (0.90, 1.0, 0.0, 0.0, 0.0, 0.0);
  avYFact : array[1..6] of Float = (0.05, 0.0, 0.0, 1.0, 0.0, 0.0);
  avZFact : array[1..6] of Float = (0.00, 0.0, 0.0, 0.0, 0.0, 1.0);
var
  ix,iy       : integer;
//  vLargeX,vLargeY : Float;
  vXLeft,
  vXDelta,
  vYDown,
  vYDelta
        : Float;
begin
vXLeft  :=vSurfXOrg + vSurfDX * nUseMinIndexX;
vXDelta :=vSurfDX * (nUseMaxIndexX - nUseMinIndexX);
vYDown  :=vSurfYOrg + vSurfDY * nUseMinIndexY;
vYDelta :=vSurfDY * (nUseMaxIndexY - nUseMinIndexY);

// Drawing of the ground frame
if ((wFlags and flaSurf2D_DrawGroundFrame) <> 0) then begin
   iline:=1;
   for ix:=0 to 1 do for iy:=0 to 1 do begin
       projectInt(vXLeft + ix*vXDelta, vYDown + iy*vYDelta, 0,
                   panXLine^[iline], panYLine^[iline]);
       iline:=(iline+iline) mod 5;
       end; { iline varies as 1 , 2 , 4 , 3 }

   panXLine^[5]:=panXLine^[1];
   panYLine^[5]:=panYLine^[1];

   SelectObject(hDCDraw, hPenFrame);
   position:=Hidden;
   MultiLine(5);
   end;

{ --- trace the X and Y axes --- }
if ((wFlags and flaSurf2D_DrawAxes) <> 0) then begin
   SelectObject(hDCDraw, hPenAxes);
   for ix:=1 to 6 do
      projectInt( vSurfXOrg + avXFact[ix]*vXDelta/2,
                  vSurfYOrg + avYFact[ix]*vYDelta/2,
                  avZFact[ix]*vZMax, panXLine^[ix], panYLine^[ix]);

   MultiLine(6);
   end;

end;  // TSurface2DFrame0.DrawGroundFrameAndAxes

procedure TSurface2DFrame0.DrawSurface;
//=====================================
// Drawing of the surface
var
  nxdep,
  nydep,
  nn          : integer;

begin
{ ---- limits initialisation ----- }
for nn:=0 to nXPixelMax do begin
    ymin[nn]    := 32000; { > max number of pixels in the Y axis }
    ymin_new[nn]:= 32000; { idem }
    ymax[nn]    :=-32000; {iydown}
    ymax_new[nn]:=-32000; {iydown}
    end;

{$Q-} { no overflow checking }
{ Drawing of the curves }
{ ===================== }
{ --------- first half ---------- }
for nydep:=1 to nYNbDY do begin
    i1:=0;
    i2:=0;
    j1:=nydep;
    j2:=nydep-1;
    iline:=1;

    panXLine^[iline]:=panX^[i1*nMDIY + j1];
    panYLine^[iline]:=panY^[i1*nMDIY + j1];

    XX1:=panX^[i2*nMDIY + j2];
    YY1:=panY^[i2*nMDIY + j2];
    VX1:=panX^[i1*nMDIY + j1]   - XX1;
    VY1:=panY^[i1*nMDIY + j1]   - YY1;
    VX2:=panX^[(i2+1)*nMDIY + j2] - XX1;
    VY2:=panY^[(i2+1)*nMDIY + j2] - YY1;

    if (VX1 * VY2 - VY1 * VX2) > 0 then position:=superior
    else                                position:=inferior;

    { ---- do one zig-zag line ---- }
    repeat
      draw_line;
      j1:=j2;
      i2:=i1+1;
      draw_line;
      i1:=i2;
      j2:=j1-1;
    until (j2<0) or (i2>=nXNbDX);
    if (j2>=0) then draw_line;
    if position<>hidden then MultiLine(iline);
    { ---- update ymin and ymax ---- }
    { ---- vector equality ---- }
    ymin:=ymin_new;
    ymax:=ymax_new;
    end;
{ ------- second half -------}
for nxdep:=0 to nXNbDX-1 do begin
    i1:=nxdep;
    i2:=nxdep+1;
    j1:=nYNbDY;
    j2:=nYNbDY;

    iline:=1;
    panXLine^[iline]:=panX^[i1*nMDIY + j1];
    panYLine^[iline]:=panY^[i1*nMDIY + j1];

    XX1:=panX^[i2*nMDIY + j2];
    YY1:=panY^[i2*nMDIY + j2];
    VX1:=panX^[i1*nMDIY + j1]   - XX1;
    VY1:=panY^[i1*nMDIY + j1]   - YY1;
    VX2:=panX^[i2*nMDIY + j2-1] - XX1;
    VY2:=panY^[i2*nMDIY + j2-1] - YY1;

    position:=hidden;

    if (VX1 * VY2 - VY1 * VX2) < 0 then begin
       if (panY^[i1*nMDIY + j1] <= ymin[panX^[i1*nMDIY + j1]]) then position:=superior;
       end
    else begin
       if (panY^[i1*nMDIY + j1] >= ymax[panX^[i1*nMDIY + j1]]) then position:=inferior;
       end;

    { ---- do one zig-zag line ---- }
    repeat
      draw_line;
      i1:=i2;
      j2:=j1-1;
      draw_line;
      j1:=j2;
      i2:=i1+1;
      until (i2>nXNbDX) or (j2<=0);

    if (i2<=nXNbDX) then draw_line;
    if position<>hidden then MultiLine(iline);

    { ---- update ymin and ymax ---- }
    { ---- vector equality ---- }
    ymin:=ymin_new;
    ymax:=ymax_new;
    end;
end;  // TSurface2DFrame0.DrawSurface

procedure TSurface2DFrame0.Initialise(wOptions:word32);
//=====================================================
// Should be called if a new surface or a new projection point has been
// defined.
// if wOptions = 1 then default values are initialised.
// In any case, compute the angle  vTheta and vPhi from the projection point.
var
  vNorme      : Float;
begin
if (wOptions and 1 = 1) then begin
   nUseMinIndexX :=nDefinedMinIndexX;
   nUseMaxIndexX :=nDefinedMaxIndexX;
   nUseMinIndexY :=nDefinedMinIndexY;
   nUseMaxIndexY :=nDefinedMaxIndexY;

   nDeltaIndexX:=1;
   nDeltaIndexY:=1;

   vSurfXOrg :=0;
   vSurfYOrg :=0;
   vSurfDX   :=1;
   vSurfDY   :=1;

   vZMinLimit :=-1e30;
   vZMaxLimit := 1e30;

   wFlags  :=0;

   ColorSuperior :=RGB(255,255,255); // white
   ColorInferior :=RGB(255,  0,  0); // red
   ColorFrame    :=RGB(  0,  0,255); // Blue
   ColorAxes     :=RGB(  0,255,  0); // LightGreen

   vProjZoomFactor:=1;

   nXProjTranslate :=0;
   nYProjTranslate :=0;
   end;

vNorme:=sqrt(sqr(vP1) + sqr(vP2) + sqr(vP3));
if (vNorme = 0) then vNorme:=1;

// Angle of projection
vTheta:=arcsin(vP3 / vNorme);
if ( vP1 <> 0) then vPhi  :=arcTan(vP2 / vP1)
else vPhi:=Pi / 2.0;

if (Cos(vTheta) * Sin(vPhi) * vP2 < 0) then vPhi:=vPhi + Pi;
end;  // TSurface2DFrame0.Initialise

procedure TSurface2DFrame0.Surf2DCompute(var wErrorV:word32);
//===========================================================
var
  temp
              : integer;
  rRec        : RECT;
  fDoReleasehDCDraw : boolean;
  wSizeAllocate    : word32; // Necessary memory for the pavX and pavY Matrix array

begin
// test the validity of the inputs
TestInputValidity(wErrorV);
if (wErrorV <> 0) then exit;

LastPosition:=hidden; // memorise the last position

GetWindowRect(hWin, rRec);
ixleft :=5;
iyup   :=5;
ixright:=rRec.Right  - rRec.Left -10;
iydown :=rRec.Bottom - rRec.Top  -10;

// We want that iydown < iyup
if (iydown >= iyup) then begin
   temp:=iydown;
   iydown:=iyup;
   iyup:=temp;

   if (iydown >= iyup) or (ixleft >= ixright) then begin
      wErrorV:=errSurf2D_WindowTooSmall;
      exit;
      end;
   end;

// Center of the window
ixmed:=(ixright+ixleft) div 2;
iymed:=(iyup+iydown) div 2;

// Memory allocation for the integer vectors used with MultiLine
wSizeAllocate:=(4 + nDefinedMaxIndexY - nDefinedMinIndexY
                  + nDefinedMaxIndexX - nDefinedMinIndexX) * SizeOf(integer);
if (wSizeVectInt < wSizeAllocate) then begin
   if (wSizeVectInt > 0) then begin
      // first must desallocated the too small memory allocated
      FreeMem(panXLine, wSizeVectInt);
      FreeMem(panYLine, wSizeVectInt);
      end;

   // Allocate more memory
   wSizeVectInt:=wSizeAllocate;
   GetMem(panXLine, wSizeVectInt);
   GetMem(panYLine, wSizeVectInt);
   end;

// Memory allocation to compute the projection in Float
nMDFY:=1 + nDefinedMaxIndexY - nDefinedMinIndexY;
wSizeAllocate:=word32(nMDFY) * (1 + nDefinedMaxIndexX - nDefinedMinIndexX) * SizeOf(SingleType);
if (wSizeMatFloatS < wSizeAllocate) then begin
   if (wSizeMatFloatS > 0) then begin
      // first must desallocated the too small memory allocated
      FreeMem(pavX, wSizeMatFloatS);
      FreeMem(pavY, wSizeMatFloatS);
      end;

   // Allocate more memory
   wSizeMatFloatS:=wSizeAllocate;
   GetMem(pavX, wSizeMatFloatS);
   GetMem(pavY, wSizeMatFloatS);
   end;

// Memory allocation to store the projection
nMDIY:=1 + nDefinedMaxIndexY - nDefinedMinIndexY;
wSizeAllocate:=word32(nMDIY) * (1 + nDefinedMaxIndexX - nDefinedMinIndexX) * SizeOf(integer);
if (wSizeMatInt < wSizeAllocate) then begin
   if (wSizeMatInt > 0) then begin
      // first must desallocated the too small memory allocated
      FreeMem(panX, wSizeMatInt);
      FreeMem(panY, wSizeMatInt);
      end;

   // Allocate more memory
   wSizeMatInt:=wSizeAllocate;
   GetMem(panX, wSizeMatInt);
   GetMem(panY, wSizeMatInt);
   end;

// Memory overflow test.
if (panY = NIL) or (panX = NIL) or (pavY = NIL) or (pavX = NIL) then begin
   wErrorV:=errSurf2D_MemoryOverflow;
   exit;
   end;

// Determination of the orthogonal projection on the plane p1*x+vP2*y+vP3*z=0
ComputProjectionMatrix;

// Projection of the surface on the plane and computation of the limits
SurfaceProjection;

{ ------ transformation of the coord. in integer   }
{ ------ between (ixleft,ixright) for the x-axis,  }
{ ------ between (iydown,iyup   ) for the y-axis.  }
{ ------ such that the proportions are respected   }
ConvertToInteger;

// We begin to draw, thus define a device context
fDoReleasehDCDraw:=(hDCDraw = 0);
if fDoReleasehDCDraw then hDCDraw:=GetDC(hWin);

// Create the pens colors
if (hPenSuperior = 0) then hPenSuperior:=CreatePen(PS_SOLID, 1, ColorSuperior);
if (hPenInferior = 0) then hPenInferior:=CreatePen(PS_SOLID, 1, ColorInferior);
if (hPenFrame = 0)    then hPenFrame   :=CreatePen(PS_SOLID, 1, ColorFrame);
if (hPenAxes = 0)     then hPenAxes    :=CreatePen(PS_SOLID, 1, ColorAxes);

// Select one pen, and remember the original pen of the device context
hPenOld:=SelectObject(hDCDraw, hPenFrame);

// Drawing of the ground frame
DrawGroundFrameAndAxes;

// Drawing of the surface (Most of the time is consumed here)
DrawSurface;

wErrorV:=0;

// Restore the original pen
SelectObject(hDCDraw, hPenOld);
if fDoReleasehDCDraw then begin
   ReleaseDC(hWin, hDCDraw);
   hDCDraw:=0;
   end;
end;  // TSurface2DFrame0.Surf2DCompute

procedure TSurface2DFrame0.Visualise(wPar:WPARAM);
//================================================
// To rotate around the surface
var
  rRec        : RECT;
  hBMAPOldL,
  hBMap       : HBITMAP; // BitMap in which we will draw.

  hDCMem,     // A memory device context
  hDCWin      // The device context of the client window, to draw on it.
              : HDC;
  wErrorL     : word32;

begin
if (nMatMinIndexX >= nMatMaxIndexX) then exit; // matrix not defined

case wPar of
   VK_NUMPAD4 : begin
      if (GetAsyncKeyState(VK_CONTROL) < 0) then begin
         // Translate Up
         nXProjTranslate:=nXProjTranslate - 5;
         end
      else begin
         // rotate left
         vPhi:=vPhi + 0.02;
         end;
      end;
   VK_NUMPAD6 : begin
      if (GetAsyncKeyState(VK_CONTROL) < 0) then begin
         // Translate Up
         nXProjTranslate:=nXProjTranslate + 5;
         end
      else begin
         // rotate right
         vPhi:=vPhi - 0.02;
         end;
      end;
   VK_NUMPAD8 : begin
      if (GetAsyncKeyState(VK_CONTROL) < 0) then begin
         // Translate Up
         nYProjTranslate:=nYProjTranslate - 5;
         end
      else begin
         // rotate up
         vTheta:=vTheta - 0.02;
         end;
      end;
   VK_NUMPAD2 : begin
      if (GetAsyncKeyState(VK_CONTROL) < 0) then begin
         // Translate Up
         nYProjTranslate:=nYProjTranslate + 5;
         end
      else begin
         // rotate down
         vTheta:=vTheta + 0.02;
         end;
      end;
   VK_ADD : begin
      // Zoom in
      vProjZoomFactor:=vProjZoomFactor * 1.02;
      end;
   VK_SUBTRACT : begin
      // Zoom out
      vProjZoomFactor:=vProjZoomFactor / 1.02;
      end;
   VK_NUMPAD0 : begin
      // More resolution
      nDeltaIndexX:=nDeltaIndexX-1;
      nDeltaIndexY:=nDeltaIndexY-1;
      // In the validity test, will limit the value
      end;
   VK_DECIMAL : begin
      // Less resolution
      nDeltaIndexX:=nDeltaIndexX+1;
      nDeltaIndexY:=nDeltaIndexY+1;
      // In the validity test, will limit the value
      end;
   VK_NUMPAD1 : begin
      if (GetAsyncKeyState(VK_CONTROL) < 0) then
         nUseMinIndexX:=nUseMinIndexX +1 // Cut on left
      else
         nUseMinIndexX:=nUseMinIndexX -1; // Add on left
      end;
   VK_NUMPAD3 : begin
      if (GetAsyncKeyState(VK_CONTROL) < 0) then
         nUseMaxIndexX:=nUseMaxIndexX -1 // Cut on right
      else
         nUseMaxIndexX:=nUseMaxIndexX +1; // Add on right
      end;
   VK_NUMPAD7 : begin
      if (GetAsyncKeyState(VK_CONTROL) < 0) then
         nUseMinIndexY:=nUseMinIndexY +1 // Cut on bottom
      else
         nUseMinIndexY:=nUseMinIndexY -1; // Add on bottom
      end;
   VK_NUMPAD9 : begin
      if (GetAsyncKeyState(VK_CONTROL) < 0) then
         nUseMaxIndexY:=nUseMaxIndexY -1 // Cut on top
      else
         nUseMaxIndexY:=nUseMaxIndexY +1; // Add on top
      end;
   0 : Begin end; // just to not exit.
   else exit;
   end; // case

vP3:=Sin(vTheta);
vP2:=Cos(vTheta) * Sin(vPhi);
vP1:=Cos(vTheta) * Cos(vPhi);

GetClientRect(hWin, rRec);
hDCWin:=GetDC(hWin);
hBMAP:=CreateCompatibleBitmap(hDCWin, rRec.Right, rRec.Bottom);

// Select the bitmap, to draw on it.
// You can only paint on the bit map, if it is selected in a memory device context
// Selecting it in the Window Device context will not work.
hDCMem:=CreateCompatibleDC(hDCWin);
hBMAPOldL:=SelectObject(hDCMem, hBMAP);

hDCDraw:=hDCMem;

// Fill the bitmap with a black color
FillRect(hDCDraw, rRec, GetStockObject(BLACK_BRUSH));

// Draw the surface in the memory bitmap
Surf2DCompute(wErrorL);

// Copy the bit map, from the memory DC to the paint DC
BitBlt( hDCWin, // handle of destination device context
          0, // x-coordinate of destination rectangle's upper-left corner
          0, // y-coordinate of destination rectangle's upper-left corner
        rRec.Right, // width of destination rectangle
        rRec.Bottom, // height of destination rectangle
        hDCMem, // handle of source device context
          0,  // x-coordinate of source rectangle's upper-left corner
          0, // y-coordinate of source rectangle's upper-left corner
        SRCCOPY // raster operation code  (there are many others possibilities)
        );

// You must delete the created memory device context.
// But before, you must select the original bit map which was in it.
SelectObject(hDCMem, hBMAPOldL);
DeleteDC(hDCMem);
DeleteObject(hBMAP);
hDCDraw:=0;

ReleaseDC(hWin, hDCWin);
end;  // TSurface2DFrame0.Visualise

procedure TSurface2DFrame0.EraseBackground;
//=========================================
// Errase the background of the window
var rRec        : RECT;
Begin
hDCDraw:=GetDC(hWin);
GetClientRect(hWin, rRec);
FillRect(hDCDraw, rRec, GetStockObject(BLACK_BRUSH));
ReleaseDC(hWin, hDCDraw);
hDCDraw:=0;
end;

procedure TSurface2DFrame0.WMPaint;
//=================================
var
  rPaint      : PAINTSTRUCT;
  wErrorL     : word32;
  rRec        : RECT;

Begin
// Gets the info to paint.
hDCDraw :=BeginPaint(hWin, @rPaint);

if (nMatMinIndexX < nMatMaxIndexX) then begin
   // Fill the bitmap with a black color
   GetClientRect(hWin, rRec);
   FillRect(hDCDraw, rRec, GetStockObject(BLACK_BRUSH));

   // Draw the surface
   Surf2DCompute(wErrorL);
   end;

EndPaint(hDCDraw, @rPaint);

hDCDraw:=0;
end;  //  TSurface2DFrame0.WMPaint

function TSurface2DFrame0.CtrlWindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT;
//==================================================================================================
// Handles the window messages
var
  rRec        : RECT;
begin
CtrlWindowProc:=0;

if (wMsg = WM_Paint) then begin WMPAINT; exit; end;

if (wMsg = WM_ERASEBKGND) then begin
   // Set the background to black
   GetClientRect(hWin, rRec);
   FillRect(HDC(wPar), rRec, GetStockObject(BLACK_BRUSH));
   CtrlWindowProc:=1;
   exit;
   end;

CtrlWindowProc:=inherited CtrlWindowProc(hWindow, wMsg, wPar, lPar);
end;  // TSurface2DFrame0.CtrlWindowProc

end.  // unit wSurface2D0;

